from enum import Enum
from typing import Dict

from langchain import hub
from langchain_core.prompts import BasePromptTemplate

from langchain_cfg_build.obj.cache_provider import CacheProvider

_ALL_PROMPT_MAP: Dict[str, BasePromptTemplate] = dict()


class EnumPrompt(Enum):
    HWCHASE17_REACT = CacheProvider[BasePromptTemplate]("HWCHASE17_REACT", _ALL_PROMPT_MAP,
                                                        lambda: hub.pull("hwchase17/react"))



import argparse
import os

from dotenv import load_dotenv

from langchain_cfg_build import get_root_path

_inited = False


def load_env():
    global _inited
    if _inited:
        return
    _inited = True
    load_dotenv()
    load_dotenv(verbose=True)
    parser = argparse.ArgumentParser()
    parser.add_argument("-e", "--env", help="environment")
    args = parser.parse_args()
    if not args.env:
        return
    env_path = get_root_path() / f'{args.env}.env'
    load_dotenv(dotenv_path=env_path, override=True)


def env(s) -> str:
    load_env()
    return os.getenv(s)


def env_int(s) -> int:
    load_env()
    return int(os.getenv(s))


def env_float(s) -> float:
    load_env()
    return float(os.getenv(s))


def env_bool(s) -> bool:
    load_env()
    return env(s) == 'true'


if __name__ == '__main__':
    load_env()
    print(env('db_port'))



import os
import shutil


def delete_folder_and_contents(folder_path: str):
    if os.path.exists(folder_path) and os.path.isdir(folder_path):
        shutil.rmtree(folder_path)
        print(f"Successfully deleted the folder and all its contents: {folder_path}")
    else:
        print(f"The folder does not exist or is not a directory: {folder_path}")


import os
from typing import Iterator, List

from langchain_chroma import Chroma
from langchain_core.documents import Document
from langchain_openai import OpenAIEmbeddings

from langchain_cfg_build.infra import config
from langchain_cfg_build.utils import file_utils


def save_embed_db(db_path: str, doc_iterator: Iterator[List[Document]]) -> str:
    exist_db_folder = os.path.exists(db_path)
    if exist_db_folder:
        file_utils.delete_folder_and_contents(db_path)
    embeddings = OpenAIEmbeddings()
    db = Chroma(persist_directory=db_path, embedding_function=embeddings)
    for doc_list in doc_iterator:
        db.add_documents(doc_list)
    return db_path


def _test_document_generator() -> Iterator[List[Document]]:
    # Imagine this function pulls documents from a database or some other source
    for i in range(20):  # Example of generating 1000 batches
        print(f"Generating {i}")
        yield [Document(
            page_content=f"1This is document {i+1}. It contains some sample text.",
            metadata={"source_url": f"https://1example.com/document/{i+1}"}
        ), Document(
            page_content=f"1This is another document {i+1}. It contains more sample text.",
            metadata={"source_url": f"https://2example.com/document/{i+1}"}
        )]


if __name__ == '__main__':
    config.load_env()
    _test_it = _test_document_generator()
    print(_test_it)
    wd_path = os.path.dirname(__file__)
    db_path = wd_path + '/_db'
    db_path = save_embed_db(db_path, _test_it)


from typing import Callable, TypeVar, Generic, Dict

T = TypeVar("T")


class CacheProvider(Generic[T]):

    def __init__(self, name: str, cache_map: Dict[str, T], provide: Callable[[], T]):
        self.name: str = name
        self.cache_map: Dict[str, T] = cache_map
        self.provide: Callable[[], T] = provide

    def get_instance(self) -> T:
        if self.name not in self.cache_map:
            self.cache_map[self.name] = self.provide()
        return self.cache_map[self.name]


# ti = CacheProvider[str]("cache", [], lambda: "1233")



from typing import Dict, List, Callable

from langchain_core.tools import BaseTool

from langchain_cfg_build.obj.cache_provider import CacheProvider

_ALL_TOOL_MAP: Dict[str, BaseTool] = dict()


class ToolRepository:
    def __init__(self):
        self._provider_list: List[CacheProvider[BaseTool]] = list()

    def add_tool(self, tool_name: str, provide: Callable[[], BaseTool]):
        self._provider_list.append(CacheProvider[BaseTool](tool_name, _ALL_TOOL_MAP, provide))


instance: ToolRepository = None


def get_tool_repo():
    global instance
    if not instance:
        instance = ToolRepository()
    return instance


if __name__ == '__main__':
    print('Testing')



from enum import Enum
from typing import Dict

from langchain_openai.chat_models import ChatOpenAI
from langchain_core.language_models import BaseLanguageModel

from langchain_cfg_build.obj.cache_provider import CacheProvider

_ALL_LLM_MAP: Dict[str, BaseLanguageModel] = dict()


class EnumLLM(Enum):
    gpt_4o = CacheProvider[BaseLanguageModel]("gpt_4o", _ALL_LLM_MAP, lambda: ChatOpenAI(model_name="gpt-4"))
    # gpt_4o: BaseLanguageModel = ChatOpenAI(model_name="gpt-4")


def get_enum_by_name(name: str) -> EnumLLM:
    return EnumLLM[name]


if __name__ == '__main__':
    name = "gpt_4o"
    enum_member = get_enum_by_name(name)
    print(enum_member)
    print(enum_member.value)
